@@ -1,7 +1,42 @@
-/* This program, double, is a filter for converting, scaling, cutting and
-   joining data sets.  The data sets may be unformatted (i.e., in binary
-   double precision format), or in ascii format.  Copyright (C) 1989-1998
-   Free Software Foundation, Inc. */
+/* This program, double, is a filter for converting, scaling, joining, and
+   cutting data sets that are in the format accepted by the `spline' and
+   `graph' utilities.  The data sets may be in binary (either double or
+   single precision floating point format, or integer format), or in ascii.
+   Copyright (C) 1989-1998 Free Software Foundation, Inc.
+
+   The `I', `O', and `q' options (i.e. `--input-type', `--output-type', and
+   `--precision') are similar to those accepted by `spline' and `graph'.
+   For example, `-I a' specifies that the input is in ascii format, and 
+   `-O i' that the output should be in binary integer format.
+   The length of each record in the input is specified with the `-R' option.
+   For example, `-R 2' would be appropriate if the input consists of
+   pairs of numbers (x,y).  The default record length is 1. 
+
+   By default, `double' copies all fields of each input record to a
+   corresponding output record.  Since `-R 1' is the default, without
+   additional options it will simply copy an input stream of numbers to an
+   output stream.  You can use the `-f' option to specify which fields you
+   want copied.  For example, `-R 3 -f 2 0' would interpret the input as
+   being made of size-3 records, and would produce output consisting of
+   size-2 records, each of which would be made from field #2 and field #0
+   of the corresponding input record.  (Fields are numbered starting with
+   zero.)  In the `-f' specification, fields may be repeated.
+
+   You can use the `-m' option to multiply all the input fields by a
+   constant, and `-p' to add a constant to each of the input fields.
+
+   `double' can also join streams of records together.  The `-j' and `-J'
+   options are used to specify the names of files consisting of size-1
+   records, called a `pre-file' and a `post-file'.  If you use `-j
+   filename' in the above example, the output records will be of size 3
+   rather than size 2, and the first field of each output record will be
+   taken from the corresponding record in `filename'.  So the `-j' option
+   `prepends' a component to each record.  Similarly, `-J' will append a
+   component.
+
+   `double' does not require that its input file(s) consist of only a
+   single dataset.  However, the lengths of the corresponding datasets
+   should be equal. */
 
 #include "sys-defines.h"
 #include "getopt.h"
@@ -18,6 +53,9 @@ data_type output_type = T_ASCII;
 
 const char *progname = "double"; /* name of this program */
 
+const char *usage_appendage = " [FILE]...\n\
+With no FILE, or when FILE is -, read standard input.\n";
+
 int precision = 8;		/* default no. of digits after decimal pt. */
 
 #define	ARG_NONE	0
@@ -53,7 +91,7 @@ struct option long_options[] =
 int hidden_options[] = { 0 };
 
 /* forward references */
-bool mung_dataset __P ((FILE *input, int record_length, bool *field_array, double scale, double baseline, FILE *add_fp, FILE *mult_fp, FILE *pre_join_fp, FILE *post_join_fp, int precision, bool suppress));
+bool mung_dataset __P ((FILE *input, int record_length, int *field_array, int field_array_len, double scale, double baseline, FILE *add_fp, FILE *mult_fp, FILE *pre_join_fp, FILE *post_join_fp, int precision, bool suppress));
 bool read_float __P ((FILE *input, double *dptr));
 bool skip_whitespace __P ((FILE *stream));
 bool write_float __P ((double data, int precision));
@@ -63,10 +101,11 @@ void open_file __P ((char *name, FILE **fpp));
 void output_dataset_separator __P ((void));
 void set_format_type __P ((char *s, data_type *typep));
 /* from libcommon */
-extern void display_usage __P((const char *progname, const int *omit_vals, bool files, bool fonts));
+extern void display_usage __P((const char *progname, const int *omit_vals, const char *appendage, bool fonts));
 extern void display_version __P((const char *progname)); 
 extern Voidptr xcalloc __P ((unsigned int nmemb, unsigned int size));
-extern Voidptr xmalloc __P ((unsigned int size));
+extern Voidptr xmalloc __P ((unsigned int length));
+extern Voidptr xrealloc __P ((Voidptr p, unsigned int length));
 extern char *xstrdup __P ((const char *s));
 
 
@@ -79,9 +118,10 @@ main (argc, argv)
      char **argv;
 #endif
 {
-  int option;
+  int option;			/* for option parsing */
   int opt_index;
-  int errcnt = 0;		/* errors encountered */
+  int errcnt = 0;		/* errors encountered in parsing */
+  int i;
   bool show_version = false;	/* remember to show version message */
   bool show_usage = false;	/* remember to output usage message */
   char *add_file = NULL, *mult_file = NULL;
@@ -89,14 +129,14 @@ main (argc, argv)
   FILE *add_fp = NULL, *mult_fp = NULL; 
   FILE *pre_join_fp = NULL, *post_join_fp = NULL;
   double scale = 1.0, baseline = 0.0; /* mult., additive constants */
-  int record_length = 1;
-  int record_min = 0, record_max = MAXINT, record_spacing = 1;
-  int dataset_min = 0, dataset_max = MAXINT, dataset_spacing = 1;  
+  int record_length = 1;	/* default record length */
+  int record_min = 0, record_max = INT_MAX, record_spacing = 1;
+  int dataset_min = 0, dataset_max = INT_MAX, dataset_spacing = 1;  
   int local_record_min, local_record_max, local_record_spacing;
   int local_dataset_min, local_dataset_max, local_dataset_spacing;  
-  int field_array_len = 0;	/* initial size of field array */
+  int *field_array = NULL;	/* array of indices we'll extract */
+  int field_array_len = 0;	/* initial size of field_array[] */
   int dataset_index = 0;	/* running count */
-  bool *field_array = NULL, *new_field_array;
   bool more_points, dataset_printed = false;
 
   while (true)
@@ -237,19 +277,14 @@ main (argc, argv)
 			   progname, field_index);
 		  return 1;
 		}
-	      if (field_index >= field_array_len)
-		{
-		  new_field_array 
-		    = (bool *)xcalloc ((unsigned)(field_index + 1), sizeof(bool));
-		  memcpy (new_field_array, field_array,
-			  field_array_len * sizeof(bool));
-		  if (field_array != NULL) /* NULL on entry */
-		    free (field_array);
-		  field_array = new_field_array;
-		  field_array_len = field_index + 1;
-		}
-
-	      field_array[field_index] = true;
+	      if (field_array_len == 0)
+		field_array = 
+		  (int *)xmalloc ((++field_array_len) * sizeof(int));
+	      else
+		field_array = 
+		  (int *)xrealloc (field_array, 
+				   (++field_array_len) * sizeof(int));
+	      field_array[field_array_len - 1] = field_index;
 	      optind++;		/* tell getopt we recognized field index */
 	    }
 	  break;
@@ -280,10 +315,12 @@ main (argc, argv)
     }
   if (show_usage)
     {
-      display_usage (progname, hidden_options, true, false);
+      display_usage (progname, hidden_options, usage_appendage, false);
       return 0;
     }
 
+  /* Sanity checks on user-supplied options */
+
   if (dataset_spacing < 1)
     {
       fprintf (stderr, "%s: error: bad dataset spacing `%d' (must be > 0)\n",
@@ -298,25 +335,25 @@ main (argc, argv)
       return 1;
     }
 
-  /* Sanity checks on user-supplied options */
+  for (i = 0; i < field_array_len; i++)
+    if (field_array[i] >= record_length)
+      {
+	fprintf (stderr, 
+		 "%s: error: at least one field index out of bounds\n", progname);
+	return 1;
+      }
 
+  /* default if no `-R' option seen: extract all fields of each record */
   if (field_array_len == 0)
-    fprintf (stderr, "%s: no fields specified\n", progname);
-  if (field_array_len > record_length)
     {
-      fprintf (stderr, 
-	       "%s: error: at least one field index out of bounds\n", progname);
-      return 1;
+      field_array = 
+	(int *)xmalloc ((record_length) * sizeof(int));
+      field_array_len = record_length;
+      for (i = 0; i < field_array_len; i++)
+	field_array[i] = i;
     }
-  new_field_array = (bool *)xcalloc ((unsigned)record_length, sizeof(bool));
-  memcpy (new_field_array, field_array,
-	  field_array_len * sizeof(bool));
-  if (field_array != NULL) /* NULL originally */
-    free (field_array);
-  field_array = new_field_array;
-  field_array_len = record_length;
 
-  /*  open additive/multiplicative/join files. */
+  /* open additive/multiplicative/join files. */
   if (add_file)
     open_file (add_file, &add_fp);
   if (mult_file)
@@ -355,7 +392,8 @@ main (argc, argv)
 		output_dataset_separator();
 
 	      more_points = mung_dataset (data_fp,
-					  record_length, field_array,
+					  record_length, 
+					  field_array, field_array_len,
 					  scale, baseline,
 					  add_fp, mult_fp, 
 					  pre_join_fp, post_join_fp,
@@ -369,8 +407,12 @@ main (argc, argv)
 	  while (more_points);
 	  
 	  /* close file (but don't close stdin) */
-	  if (data_fp != stdin)
-	    fclose (data_fp);
+	  if (data_fp != stdin && fclose (data_fp) < 0)
+	    {
+	      fprintf (stderr, "%s: error: couldn't close input file\n", 
+		       progname);
+	      return 1;
+	    }
 	}
     }
   else			/* no files spec'd, read stdin instead */
@@ -389,7 +431,8 @@ main (argc, argv)
 	  output_dataset_separator();
 
 	more_points = mung_dataset (stdin,
-				    record_length, field_array,
+				    record_length, 
+				    field_array, field_array_len,
 				    scale, baseline,
 				    add_fp, mult_fp, 
 				    pre_join_fp, post_join_fp,
@@ -461,7 +504,7 @@ read_float (input, dptr)
    read (i.e. EOF or garbage in stream).  A return value of 2 is special:
    it indicates that an explicit end-of-dataset indicator was seen in the
    input file.  For an ascii stream this is two newlines in succession;
-   for a stream of doubles it is a MAXDOUBLE appearing at what would
+   for a stream of doubles it is a DBL_MAX appearing at what would
    otherwise have been the beginning of the record, etc. */
 
 int
@@ -475,7 +518,9 @@ get_record (input, record, record_length)
 #endif
 {
   bool success;
-  int i;
+  int i, items_read, lookahead;
+
+ head:
 
   if (input_type == T_ASCII)
     {
@@ -490,15 +535,35 @@ get_record (input, record, record_length)
   if (feof (input))
     return 1;			/* EOF */
 
+  if (input_type == T_ASCII)
+    {
+      lookahead = getc (input);
+      ungetc (lookahead, input);
+      if (lookahead == (int)'#')	/* comment line */
+	{
+	  char c;
+	  
+	  do 
+	    {
+	      items_read = fread (&c, sizeof (c), 1, input);
+	      if (items_read <= 0)
+		return 1;	/* EOF */
+	    }
+	  while (c != '\n');
+	  ungetc ((int)'\n', input); /* push back \n at the end of # line */
+	  goto head;
+	}
+    }
+
   for (i = 0; i < record_length; i++)
     {
       double val;
 
       success = read_float (input, &val);
       if (i == 0 && 
-	  ((input_type == T_DOUBLE && val == MAXDOUBLE)
-	   || (input_type == T_SINGLE && val == (double)MAXFLOAT)
-	   || (input_type == T_INTEGER && val == (double)MAXINT)))
+	  ((input_type == T_DOUBLE && val == DBL_MAX)
+	   || (input_type == T_SINGLE && val == (double)FLT_MAX)
+	   || (input_type == T_INTEGER && val == (double)INT_MAX)))
 	/* end-of-dataset indicator */
 	return 2;
       if (!success)		/* EOF or garbage */
@@ -534,14 +599,14 @@ write_float (x, precision)
     {
     case T_ASCII:
     default:
-      num_written = printf ("%.*g\n", precision, x);
+      num_written = printf ("%.*g ", precision, x);
       break;
     case T_SINGLE:
       fx = FROUND(x);
-      if (fx == MAXFLOAT || fx == -(MAXFLOAT))
+      if (fx == FLT_MAX || fx == -(FLT_MAX))
 	{
 	  maybe_emit_oob_warning();
-	  if (fx == MAXFLOAT)
+	  if (fx == FLT_MAX)
 	    fx *= 0.99999;	/* kludge */
 	}
       num_written = fwrite ((Voidptr) &fx, sizeof (fx), 1, stdout);
@@ -551,10 +616,10 @@ write_float (x, precision)
       break;
     case T_INTEGER:
       ix = IROUND(x);
-      if (ix == MAXINT || ix == -(MAXINT))
+      if (ix == INT_MAX || ix == -(INT_MAX))
 	{
 	  maybe_emit_oob_warning();
-	  if (ix == MAXINT)
+	  if (ix == INT_MAX)
 	    ix--;
 	}
       num_written = fwrite ((Voidptr) &ix, sizeof (ix), 1, stdout);
@@ -580,7 +645,7 @@ open_file (name, fpp)
   fp = fopen (name, "r");
   if (fp == NULL)
     {
-      fprintf (stderr, "%s: error: couldn't open file `%s'\n", progname, name);
+      fprintf (stderr, "%s: %s: %s\n", progname, name, strerror(errno));
       exit (1);
     }
   *fpp = fp;
@@ -599,18 +664,26 @@ set_format_type (s, typep)
     {
     case 'a':
     case 'A':
+      /* ASCII format: records and fields within records are separated by
+	 whitespace, and datasets are separated by a pair of newlines. */
       *typep = T_ASCII;
       break;
     case 'f':
     case 'F':
+      /* Binary single precision: records and fields within records are
+	 contiguous, and datasets are separated by a FLT_MAX.  */
       *typep = T_SINGLE;
       break;
     case 'd':
     case 'D':
+      /* Binary double precision: records and fields within records are
+	 contiguous, and datasets are separated by a DBL_MAX. */
       *typep = T_DOUBLE;
       break;
     case 'i':
     case 'I':
+      /* Binary integer: records and fields within records are contiguous,
+	 and datasets are separated by an occurrence of INT_MAX. */
       *typep = T_INTEGER;
       break;
     default:
@@ -628,18 +701,20 @@ set_format_type (s, typep)
    records in the input file ended with an explicit end-of-dataset
    indicator, i.e., whether another dataset is expected to follow.  An
    end-of-dataset indicator is two newlines in succession for an ascii
-   stream, and a MAXDOUBLE for a stream of doubles, etc. */
+   stream, and a DBL_MAX for a stream of doubles, etc. */
 bool
 #ifdef _HAVE_PROTOS
-mung_dataset (FILE *input, int record_length, bool *field_array, 
+mung_dataset (FILE *input, int record_length, 
+	      int *field_array, int field_array_len,
 	      double scale, double baseline, FILE *add_fp, FILE *mult_fp, 
 	      FILE *pre_join_fp, FILE *post_join_fp, int precision, 
 	      bool suppress)
 #else
-mung_dataset (input, record_length, field_array, scale, baseline, add_fp, mult_fp, pre_join_fp, post_join_fp, precision, suppress)
+mung_dataset (input, record_length, field_array, field_array_len, scale, baseline, add_fp, mult_fp, pre_join_fp, post_join_fp, precision, suppress)
      FILE *input;
      int record_length;		/* number of fields per record in dataset */
-     bool *field_array;	/* fields we'll extract */
+     int *field_array;		/* array of fields we'll extract */
+     int field_array_len;	/* length of this array */
      double scale, baseline;
      FILE *add_fp, *mult_fp, *pre_join_fp, *post_join_fp;
      int precision;
@@ -690,26 +765,28 @@ mung_dataset (input, record_length, field_array, scale, baseline, add_fp, mult_f
 	  if (pre_join_fp)
 	    write_float (pre_join_data, precision);
 
-	  for (i = 0; i < record_length; i++)
-	    if (field_array[i] == true)
-	      {
-		double datum;
-		
-		datum = record[i];
-		if (mult_fp)
-		  datum *= mult_data;
-		if (add_fp)
-		  datum += add_data;
-		datum *= scale;
-		datum += baseline;
-
-		/* output the munged datum */
-		write_float (datum, precision);
-	      }
+	  for (i = 0; i < field_array_len; i++)
+	    {
+	      double datum;
+	      
+	      datum = record[field_array[i]];
+	      if (mult_fp)
+		datum *= mult_data;
+	      if (add_fp)
+		datum += add_data;
+	      datum *= scale;
+	      datum += baseline;
+	      
+	      /* output the munged datum */
+	      write_float (datum, precision);
+	    }
 	      
 	  if (post_join_fp)
 	    write_float (post_join_data, precision);
 
+	  if (output_type == T_ASCII) /* end each record with a newline */
+	    printf ("\n");
+
 	  break;
 	case 1:			/* no more records, EOF seen */
 	  return false;
@@ -755,7 +832,7 @@ skip_whitespace (stream)
 /* Output a separator between datasets.  For ascii-format output streams
    this is an extra newline (after the one that the spline ended with,
    yielding two newlines in succession).  For double-format output streams
-   this is a MAXDOUBLE, etc. */
+   this is a DBL_MAX, etc. */
 
 void
 #ifdef _HAVE_PROTOS
@@ -775,15 +852,15 @@ output_dataset_separator()
       printf ("\n");
       break;
     case T_DOUBLE:
-      ddummy = MAXDOUBLE;
+      ddummy = DBL_MAX;
       fwrite ((Voidptr) &ddummy, sizeof(ddummy), 1, stdout);
       break;
     case T_SINGLE:
-      fdummy = MAXFLOAT;
+      fdummy = FLT_MAX;
       fwrite ((Voidptr) &fdummy, sizeof(fdummy), 1, stdout);
       break;
     case T_INTEGER:
-      idummy = MAXINT;
+      idummy = INT_MAX;
       fwrite ((Voidptr) &idummy, sizeof(idummy), 1, stdout);
       break;
     }
