@@ -163,22 +163,21 @@ _g_fspace2 (x0, y0, x1, y1, x2, y2)
      space().  So when the user calls space() [or space2() or fspace() or
      fspace2()], we set the line width and font size to something reasonable.
 
-     We also set the default line width and font size, as stored in the
-     _plotter->default_drawstate structure, to something reasonable.  The
-     default values stored there will be used later, if the user calls
-     linewidth() or fontsize() with out-of-bound arguments. */
+     We also set the default line width and font size to something
+     reasonable.  The default values stored there are used if the user
+     calls linewidth() or fontsize() with out-of-bound arguments. */
 
   /* Incidentally, invoking ffontsize will invoke the internal
      retrieve_font() method.  So e.g. if the Plotter is an XPlotter then
      invoking ffontsize() will retrieve an X font from the X server. */
 
-  _plotter->default_drawstate->line_width 
+  _plotter->drawstate->default_line_width 
     = DEFAULT_LINE_WIDTH_AS_FRACTION_OF_DISPLAY_WIDTH / norm;
-  _plotter->default_drawstate->font_size
+  _plotter->drawstate->default_font_size
     = DEFAULT_FONT_SIZE_AS_FRACTION_OF_DISPLAY_WIDTH / norm;
 
-  _plotter->flinewidth (_plotter->default_drawstate->line_width);
-  _plotter->ffontsize (_plotter->default_drawstate->font_size);
+  _plotter->flinewidth (_plotter->drawstate->default_line_width);
+  _plotter->ffontsize (_plotter->drawstate->default_font_size);
 
   return 0;
 }
